fl

{
Pressure Sensor LPS25H

2015/09/02 15:16:06


}

\ =========================================================================== 
\ Constants 
\ =========================================================================== 
\ Slave addres h5C (SA0:GND) 
hB8 wconstant LPS25H

\ register name
8 wconstant REP_P_XL
9 wconstant REP_P_L
hA wconstant REP_P_H
hF wconstant WHO_AM_I
h10 wconstant RES_CONF
h20 wconstant CTRL_REG1
h21 wconstant CTRL_REG2
h22 wconstant CTRL_REG3
h23 wconstant CTRL_REG4
h24 wconstant INT_CFG
h25 wconstant INT_SOURCE
h27 wconstant STATUS_REG
h28 wconstant PRESS_POUT_XL
h29 wconstant PRESS_POUT_L
h2A wconstant PRESS_POUT_H
h2B wconstant TEMP_OUT_L
h2C wconstant TEMP_OUT_H
h2E wconstant FIFO_CTRL
h2F wconstant FIFO_STATUS
h30 wconstant THS_P_L
h31 wconstant THS_P_H
h39 wconstant RPDS_L
h3A wconstant RPDS_H

\ bit data
0 wconstant P8
1 wconstant P32
2 wconstant P128
3 wconstant P512
0 wconstant T8
4 wconstant T16
8 wconstant T32
hC wconstant T64
4 wconstant reset
h80 wconstant pwr_act
h40 wconstant FIFO_EN
0 wconstant 1shot
h10 wconstant 1Hz
h20 wconstant 7Hz
h30 wconstant 12.5Hz
h40 wconstant 25Hz
1 wconstant start_1shot
\ Mode
0 wconstant Bypass
hC0 wconstant FIFO_mean_Mode
\ WTM
1 wconstant Sample2
3 wconstant Sample4
7 wconstant Sample8
hF wconstant Sample16
h1F wconstant Sample32

\ =========================================================================== 
\ Variables 
\ =========================================================================== 

\ =========================================================================== 
\ Main
\ =========================================================================== 

\ Check communiation
\ If reply is "BD", chip is normal.
\ ( -- )
: chkID 1 WHO_AM_I LPS25H i2c_rd_multi err? hBD =
if ." Correct  deviceID:hBD"
else ." Incorrect"
then
cr
;

\ TAB
\ ( -- )
: tab 9 emit ;

\ Read 1 byte
\ ( n1 -- n2 ) n1:register  n2:value
: rdByte LPS25H i2c_rd ;

\ Write 1 byte
\ ( n1 n2 -- )  n1:value n2:register
: wrByte LPS25H i2c_wr ;

\ Set bit
\ ( n1 n2 -- ) n1:bit data  n2:register
: setbit
2dup                     \ ( n1 n2 n1 n2 ) 
rdByte or                \ ( n1 n2 value )
swap                     \ ( n1 value n2 )
LPS25H i2c_wr            \ ( n1 )
drop
;


\ Display byte
\ ( -- )
: dispByte rdByte . cr ;

\ Display all registers
\ ( -- )
: disp_reg
." Register" tab ." Value" cr
hex
hB 8 do tab i . tab i dispByte loop
h11 hF do tab i . tab i dispByte loop
h26 h20 do tab i . tab i dispByte loop
h2D h27 do tab i . tab i dispByte loop
h32 h2E do tab i . tab i dispByte loop
h3B h39 do tab i . tab i dispByte loop
decimal
;

\ Turn on device
\ ( -- )
: active pwr_act CTRL_REG1 setbit ;

\ Set output rate
\ ( n1 -- )  n1:output rate for Pressure and Temperature 
: setRate
\ Read 1byte and clear bit[6-4]
CTRL_REG1 rdByte h8F and
\ Write new value 
or CTRL_REG1 wrByte
;

\ Set internal average for Pressure and Temperature
\ ( n1 n2 -- )  n1:pressure  n2:temperature
: setRes
or                            \ Or press and temp
RES_CONF rdByte hF0 and       \ Clear bit[3..0] 
or RES_CONF wrByte
;

\ Start oneshot measurement
\ ( -- )
: start_OS start_1shot CTRL_REG2 setbit ;

\ Set mode
\ ( n1 -- ) n1:mode
: setMode
FIFO_CTRL rdByte h1F and 
or FIFO_CTRL wrByte
;

\ Set WTM_POINT
\ ( n1 -- ) n1:WTM_POINT
: setWTM
FIFO_CTRL rdByte hE0 and 
or FIFO_CTRL wrByte
;


\ OneShot Mode
\ Pressure internal average:512   
\ Temperatire internal average:64
\ ( -- )
: measure_1shot
T64 P512 setRes
1shot setRate
active

begin
     \ Start one-shot measurement
     start_OS
     \ Wait until measurement finish
     begin CTRL_REG2 rdByte 1 and 0= until
     h2D h28 do i rdByte loop        
     \ Temperature              
     8 lshift or dup h8000 and      
     if
          hFFFF0000 or 
     then
     d48 / d425 +
     d10 u/mod . h2E emit . ." degree"    
     tab 
     
     \ Pressure
     d16 lshift swap 8 lshift or or d41 / . ." Pa" 
     tab
     cr
     d1000 delms
     fkey? swap drop
until     
;

\ FIFO mean Mode
\ Pressure internal average:32      
\ Temperatire internal average:16
\ ( -- )
: measure_FIFO
T16 P32 setRes
FIFO_mean_Mode setMode
Sample2 setWTM
FIFO_EN CTRL_REG2 setbit
1Hz setRate
active

begin
     \ Temperature
     h2D h2B do i rdByte loop
     8 lshift or dup h8000 and      
     if
          hFFFF0000 or 
     then
     d48 / d425 +
     d10 u/mod . h2E emit . ." degree"    
     tab 
     \ Pressure
     h2B h28 do i rdByte loop 
     d16 lshift swap 8 lshift or or d41 / . ." Pa" 
     tab
     cr
     d1000 delms
     fkey? swap drop
until     
;

